/*
	GeneratePreviewForURL.m
        Copyright 2008 Laura Dickey.  All rights reserved.

        This file is part of BrushViewQL.

        BrushViewQL is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        BrushViewQL is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with BrushViewQL; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330,
        Boston, MA  02111-1307  USA

        You may contact the author at: brushviewql@gmail.com
*/

#include <CoreFoundation/CoreFoundation.h>
#include <CoreServices/CoreServices.h>
#include <QuickLook/QuickLook.h>
#include <Foundation/Foundation.h>
#include "BrushFile.h"

#define DIMENSIONS_HEIGHT	20.0
#define SPACING_WIDTH		10.0
#define SPACING_HEIGHT		5.0


void BRLog(NSString *format, ...)
{
#ifdef DEBUG
        va_list ap;
        va_start(ap, format);
        NSLogv(format, ap);
        va_end(ap);
#endif
}

void
DrawBrushes(BrushFile *brushFile, NSGraphicsContext *context, float brushSize,
	NSSize canvasSize);
	
/* -----------------------------------------------------------------------------
   Generate a preview for file

   This function's job is to create preview for designated file
   ----------------------------------------------------------------------------- */

OSStatus GeneratePreviewForURL(void *thisInterface, QLPreviewRequestRef preview, CFURLRef url, CFStringRef contentTypeUTI, CFDictionaryRef options)
{
    
	NSString	*fileName;
	BrushFile	*theBrush;
	NSSize		canvasSize;
	int		numBrushes;
	float		brushSize;
	CGContextRef	cgContext;
	int		rows, cols;
	
	NSGraphicsContext	*context;
	
	fileName = [(NSURL *)url path];
	theBrush = [[BrushFile alloc] init];
	if (![theBrush parseBrushFile:fileName]) {
		return noErr; // wrong, but live with it for now
	}
	brushSize = 200.0;
	numBrushes = [theBrush numBrushes];
	if (numBrushes == 0) {
			// avoid a divide by 0 problem later
		return noErr;
	}
	cols = ceilf(sqrt(numBrushes));
	if (cols == 0) {
		cols = 1;
	}
	rows = numBrushes / cols;
	if ((rows * cols) < numBrushes) {
		++rows;
	}
	canvasSize = NSMakeSize((brushSize + SPACING_WIDTH) * cols, 
			(brushSize + DIMENSIONS_HEIGHT + SPACING_HEIGHT) * rows);
	
	BRLog(@"asking for canvas size: %f x %f",
		canvasSize.width, canvasSize.height);
	cgContext = QLPreviewRequestCreateContext(
			preview, *(CGSize *)&canvasSize, true, NULL);
	if (cgContext) {
		context = [NSGraphicsContext graphicsContextWithGraphicsPort:
			(void *)cgContext flipped:NO];
		if (context) {
			DrawBrushes(theBrush, context, brushSize, canvasSize);
		}
		QLPreviewRequestFlushContext(preview, cgContext);
		CFRelease(cgContext);
	}

	return noErr;
}

void
DrawBrushes(BrushFile *brushFile, NSGraphicsContext *context, float brushSize,
	NSSize canvasSize)
{
    // Drawing code here.
	NSImage	*myImage;
	NSArray	*images;
	NSString	*sizeStr;
	NSDictionary	*strAttribs;
	NSPoint		strLoc;
	float	totalWidth, totalHeight, maxHeight, maxWidth, maxLineHeight;
	float	currPosX, currPosY, scaleFactor, scaleX, scaleY;
	NSSize	mySize, strSize;
	int	numBrushes;
	int	cols, rows;
	
	//NSRect	myFrame, myBounds;
	
	BRLog(@"ImageView drawRect");

	images = [brushFile getImages];
	if (images == nil) {
		BRLog(@"No Images");
		return;
	}

	[NSGraphicsContext saveGraphicsState];
	[NSGraphicsContext setCurrentContext:context];

	totalWidth = 0.0;
	maxHeight = 0.0;
	maxWidth = 0.0;
	mySize = canvasSize;

	strAttribs = [NSDictionary dictionaryWithObjectsAndKeys:
		[NSFont systemFontOfSize:16], NSFontAttributeName,
		[NSColor whiteColor], NSForegroundColorAttributeName,
		nil];
	
	if (brushSize > mySize.width) {
		cols = 1;
	} else {
		cols = (int)(mySize.width / brushSize);
	}
	if (cols == 0) {
		cols = 1;
	}
	numBrushes = [images count];
	rows = numBrushes / cols;
	if ((numBrushes % cols) != 0) {
		++rows;
	}
	BRLog(@"cols = %d, rows = %d", cols, rows);
	for (myImage in images) {
		totalWidth += [myImage size].width;
		totalHeight += [myImage size].height;
		if ([myImage size].height > maxHeight) {
			maxHeight = [myImage size].height;
		}
		if ([myImage size].width > maxWidth) {
			maxWidth = [myImage size].width;
		}
	}
	
	BRLog(@"totalWidth = %f", totalWidth);
	currPosX = 0;
	currPosY = mySize.height;
	
	totalWidth = 0.0;
	maxLineHeight = 0.0;
	for (myImage in images) {
		NSSize	imgSize = [myImage size];
		NSRect	centerRect;
		
		scaleX = 1.0;
		scaleY = 1.0;
		scaleFactor = 1.0;
		
		if ((currPosX + brushSize) > mySize.width) {
			currPosX = 0;
			currPosY -= (brushSize +
					DIMENSIONS_HEIGHT + SPACING_HEIGHT);
		}
		//if (imgSize.width > brushSize) {
			scaleX = brushSize / imgSize.width;
		//}
		//if (imgSize.height > brushSize) {
			scaleY = brushSize / imgSize.height;
		//}
		if (scaleX < scaleY) {
			scaleFactor = scaleX;
		} else {
			scaleFactor = scaleY;
		}
		BRLog(@"brushSize: %f, img=(%f, %f), scale=(%f,%f), factor=%f",
			brushSize, imgSize.width, imgSize.height,
			scaleX, scaleY, scaleFactor);
		[[NSColor blackColor] set];
#if 0
		[NSBezierPath strokeRect:NSMakeRect(currPosX, (currPosY - brushSize),
			brushSize, brushSize)]; 
#endif
		
		sizeStr = [NSString stringWithFormat:@"%d x %d",
			(int)imgSize.width, (int)imgSize.height];
		strSize = [sizeStr sizeWithAttributes:strAttribs];
		strLoc.x = ((brushSize - strSize.width) / 2.0) +
			currPosX;
		strLoc.y = (currPosY - brushSize) - DIMENSIONS_HEIGHT;
		BRLog(@"strSize = (%f, %f)", strSize.width, strSize.height);

		/* center the brush inside the rectangle of brushSize */
		centerRect.origin.x = currPosX;
		centerRect.origin.x += 
			(brushSize - (scaleFactor * imgSize.width)) / 2.0;
		centerRect.origin.y = currPosY - brushSize;
		centerRect.origin.y += 
			(brushSize - (scaleFactor * imgSize.height)) / 2.0;
		centerRect.size.width = scaleFactor * imgSize.width;
		centerRect.size.height = scaleFactor * imgSize.height;
		
		[myImage drawInRect:centerRect
			fromRect:NSZeroRect 
			operation:NSCompositeCopy fraction:1.0];
		[[NSColor whiteColor] set];
		/*
		NSRectFill(NSMakeRect(currPosX, 
			(currPosY - brushSize) - DIMENSIONS_HEIGHT,
			brushSize, DIMENSIONS_HEIGHT));
		*/
		[sizeStr drawAtPoint:strLoc withAttributes:strAttribs];
		/*BRLog(@"drawing img at %f, %f", currPosX,
			currPosY - brushSize);*/
		totalWidth += [myImage size].width;
		currPosX += (brushSize + SPACING_WIDTH);
	}
	[NSGraphicsContext restoreGraphicsState];

}

void CancelPreviewGeneration(void* thisInterface, QLPreviewRequestRef preview)
{
    // implement only if supported
}
